import { CommonModule } from '@angular/common';
import { ChangeDetectorRef, Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, ReactiveFormsModule } from '@angular/forms';
import { ConfirmationService, MessageService } from 'primeng/api';
import { ButtonModule } from 'primeng/button';
import { DragDropModule } from 'primeng/dragdrop';
import { FileUploadModule } from 'primeng/fileupload';
import { OrderListModule } from 'primeng/orderlist';
import { PanelModule } from 'primeng/panel';
import { ToastModule } from 'primeng/toast';
import { first } from 'rxjs';
import { LoadingComponent } from '../../components/loading/loading.component';
import { FilesService } from '../../services/files.service';
import { RoomsService } from '../../services/rooms.service';

@Component({
  selector: 'app-add-room',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    ButtonModule,
    PanelModule,
    FileUploadModule,
    CommonModule,
    LoadingComponent,
    DragDropModule,
    ToastModule,
    OrderListModule,
  ],
  templateUrl: './add-room.component.html',
  styleUrl: './add-room.component.css',
  providers: [ConfirmationService, MessageService],
})
export class AddRoomComponent implements OnInit {
  items: any[] = [];
  isLoadingRoomImages = false;
  roomImages: any[] = [];
  isSavingRoom = false;
  form!: FormGroup;
  imageUrl: any;

  constructor(
    private fb: FormBuilder,
    private messagesService: MessageService,
    private roomsService: RoomsService,
    private cdr: ChangeDetectorRef,
    private filesService: FilesService
  ) {}

  ngOnInit(): void {
    this.form = this.fb.group({
      title: this.fb.control(''),
    });
  }

  onSelectRoomImages(e: any, fileUpload: any) {
    const arr = [];
    this.isLoadingRoomImages = true;
    this.filesService
      .uploadFiles(e.files)
      .pipe(first())
      .subscribe({
        next: (results: any) => {
          this.roomImages = [...this.roomImages, ...results];
          fileUpload.clear();
          this.isLoadingRoomImages = false;
          this.cdr.detectChanges();
        },
        error: (e: any) => {
          this.isLoadingRoomImages = false;
        },
      });
  }

  onDeleteRoomImage(id: number) {
    const idx = this.roomImages.findIndex((t) => t.id === id);
    this.roomImages.splice(idx, 1);
    this.filesService.deleteFile(id).pipe(first()).subscribe();
  }

  onSubmit() {
    this.isSavingRoom = true;
    console.log('images: ', this.roomImages);
    const data = {
      ...this.form.value,
      roomImages: this.roomImages,
    };
    this.roomsService.addRoom(data).subscribe({
      next: async (room) => {
        console.log('Room added: ', room);
        this.messagesService.add({
          severity: 'success',
          summary: 'Add',
          detail: 'Room added successfully',
        });
        this.isSavingRoom = false;
      },
      error: (e) => {
        console.log(e);
        this.messagesService.add({
          severity: 'error',
          summary: 'Error',
          detail: e.error.message,
        });
        this.isSavingRoom = false;
      },
    });
  }
}
